/*
 * Copyright 2003-2007 Sun Microsystems, Inc.  All Rights Reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
 * CA 95054 USA or visit www.sun.com if you need additional information or
 * have any questions.
 */

package sun.java2d.xr;

import static java.awt.AlphaComposite.*;

import java.awt.*;
import java.awt.MultipleGradientPaint.*;
import java.awt.image.*;

public class XRUtils {
    /* Composition Operators */
    public static final int PictOpClear = 0;
    public static final int PictOpSrc = 1;
    public static final int PictOpDst = 2;
    public static final int PictOpOver = 3;
    public static final int PictOpOverReverse = 4;
    public static final int PictOpIn = 5;
    public static final int PictOpInReverse = 6;
    public static final int PictOpOut = 7;
    public static final int PictOpOutReverse = 8;
    public static final int PictOpAtop = 9;
    public static final int PictOpAtopReverse = 10;
    public static final int PictOpXor = 11;
    public static final int PictOpAdd = 12;
    public static final int PictOpSaturate = 13;

    /* Repeats */
    public static final int RepeatNone = 0;
    public static final int RepeatNormal = 1;
    public static final int RepeatPad = 2;
    public static final int RepeatReflect = 3;

    /* Interpolation qualities */
    public static final int FAST = 0;
    public static final int GOOD = 1;
    public static final int BEST = 2;

    /* PictFormats */
    public static final int PictStandardARGB32 = 0;
    public static final int PictStandardRGB24 = 1;
    public static final int PictStandardA8 = 2;
    public static final int PictStandardA4 = 3;
    public static final int PictStandardA1 = 4;

  /**
   * Maps the specified affineTransformOp to the corresponding XRender
   * image filter.
   */
    public static int ATransOpToXRQuality(int affineTranformOp) {
	
	switch (affineTranformOp) {
	case AffineTransformOp.TYPE_NEAREST_NEIGHBOR:
	    return FAST;

	case AffineTransformOp.TYPE_BILINEAR:
	    return GOOD;

	case AffineTransformOp.TYPE_BICUBIC:
	    return BEST;
	}

	return -1;
    }

    /**
     * Returns the XRender picture Format which is 
     * required to fullfill the Java2D transparency requirement.
     */
    public static int getPictureFormatForTransparency(int transparency) {
	switch (transparency) {
	case Transparency.OPAQUE:
	    return PictStandardRGB24;

	case Transparency.BITMASK:
	case Transparency.TRANSLUCENT:
	    return PictStandardARGB32;
	}

	return -1;
    }

    /**
     * Maps the Java2D CycleMethod to XRender's Repeat properties.
     */
    public static int getRepeatForCycleMethod(CycleMethod cycleMethod) {
	if (cycleMethod.equals(CycleMethod.NO_CYCLE)) {
	    return RepeatPad;
	} else if (cycleMethod.equals(CycleMethod.REFLECT)) {
	    return RepeatReflect;
	} else if (cycleMethod.equals(CycleMethod.REPEAT)) {
	    return RepeatNormal;
	}

	return RepeatNone;
    }

    /**
     * Converts a double into an XFixed.
     */
    public static int XDoubleToFixed(double dbl) {
	return (int) (dbl * 65536);
    }

    /**
     * Maps the specified Java2D composition rule, 
     * to the corresponding XRender composition rule.
     */
    public static int j2dAlphaCompToXR(int j2dRule) {
	switch (j2dRule) {
	case CLEAR:
	    return PictOpClear;

	case SRC:
	    return PictOpSrc;

	case DST:
	    return PictOpDst;

	case SRC_OVER:
	    return PictOpOver;

	case DST_OVER:
	    return PictOpOverReverse;

	case SRC_IN:
	    return PictOpIn;

	case DST_IN:
	    return PictOpInReverse;

	case SRC_OUT:
	    return PictOpOut;

	case DST_OUT:
	    return PictOpOutReverse;

	case SRC_ATOP:
	    return PictOpAtop;

	case DST_ATOP:
	    return PictOpAtopReverse;

	case XOR:
	    return PictOpXor;
	}

	throw new InternalError("No XRender equivalent available for requested java2d composition rule");
    }
}
